package jPDFOptimizerSamples;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;
import javax.swing.text.PlainDocument;

public class IntegerRangeTextField extends JTextField
{
	public IntegerRangeTextField(int value, int min, int max, int maxDigits, String failMessage)
	{
		super(maxDigits);
		init(value, min, max, maxDigits, failMessage);
	}

	private void init(int value, int min, int max, int maxDigits, String failMessage)
	{
		setDocument(new IntegerRangeDocument(maxDigits));
		setInputVerifier(new IntegerRangeVerifier(value, min, max, failMessage));
		setText("" + value);
		addFocusListener(new FocusListener() {
			
			public void focusLost(FocusEvent e)
			{
				// TODO Auto-generated method stub
				
			}
			
			public void focusGained(FocusEvent e)
			{
				IntegerRangeTextField.this.selectAll();
			}
		});
	}
	
	public boolean verify()
	{
		return getInputVerifier().verify(this);
	}

	public class IntegerRangeDocument extends PlainDocument
	{
		static final String NUMBERS = "0123456789";
		private int maxDigits;

		public IntegerRangeDocument(int maxDigits)
		{
			this.maxDigits = maxDigits;
		}

		public void insertString(int offs, String str, AttributeSet a) throws BadLocationException
		{
			if (isValidLength(str) && isValidChars(str))
			{
				super.insertString(offs, str, a);
			}
		}

		private boolean isValidLength(String text)
		{
			if (text != null)
			{
				return this.getLength() + text.length() <= maxDigits;
			}
			return true;
		}

		private boolean isValidChars(String text)
		{
			if (text != null)
			{
				char[] chars = text.toCharArray();
				for (int i = 0; i < chars.length; i++)
				{
					if (NUMBERS.indexOf(chars[i]) == -1)
					{
						return false;
					}
				}
			}
			return true;
		}
	}

	public class IntegerRangeVerifier extends InputVerifier
	{
		private int min;
		private int max;
		private int last;
		private String failMessage;

		public IntegerRangeVerifier(int value, int min, int max, String failMessage)
		{
			this.last = value;
			this.min = min;
			this.max = max;
			this.failMessage = failMessage;
		}

		public boolean verify(JComponent input)
		{
			if (input instanceof JTextComponent)
			{
				String text = ((JTextComponent) input).getText();			
				if (text != null && text.trim().length() != 0)
				{
					int value = Integer.parseInt(text);
					boolean ret = value >= min && value <= max;
					if (!ret)
					{
						((JTextComponent) input).setText("" + last);
						SwingUtilities.invokeLater(new Runnable() {
							
							public void run()
							{
								JOptionPane.showMessageDialog(null, failMessage, null, JOptionPane.ERROR_MESSAGE);
							}
						});
						return false;
					}
					else
					{
						last = value;
					}
				}
			}
			return true;
		}
	}
}
