package jPDFOptimizerSamples;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.List;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JDialog;

import com.qoppa.pdfOptimizer.ImageHandler;
import com.qoppa.pdfOptimizer.OptSettings;


public class OptimizationOptionsController implements ActionListener, ItemListener
{
	private OptimizationOptionsDialog m_Dialog;
	private OptSettings m_Options;

	private OptimizeImagePanelController m_ColorController;
	private OptimizeImagePanelController m_GrayController;
	private OptimizeImagePanelController m_MonoController;
	
	public OptimizationOptionsController(OptSettings options)
	{
		m_Options = options;
	}

	public void showDialog(Frame frame)
	{
		m_Dialog = new OptimizationOptionsDialog(frame);
		initDialog();
		centerDialog(frame, m_Dialog);
		m_Dialog.setVisible(true);
	}

	protected static void centerDialog (Component parent, JDialog dialog)
    {
        dialog.setLocation (parent.getX () + (parent.getWidth() - dialog.getWidth()) / 2,
							parent.getY() + (parent.getHeight() - dialog.getHeight()) / 2);
    }

	private void initDialog()
	{
		initControllers();
		addListeners();

		// Mono does not have jpeg options - hide quality since it is only related to jpeg
//		m_MonoController.getPanel().getJlQuality().setVisible(false);
//		m_MonoController.getPanel().getJsQuality().setVisible(false);
//		m_MonoController.getPanel().getQhQuality().setVisible(false);
		
		initDisableImagePanels();
		loadPreferences();
	}

	// This disables the combos and spinners
	private void initDisableImagePanels()
	{
		m_ColorController.initDisable();
		m_GrayController.initDisable();
		m_MonoController.initDisable();
	}

	private void initControllers()
	{
		Vector compressionMono = new Vector();
		compressionMono.add(OptimizePanelUtil.DO_NOT_CHANGE);
		compressionMono.add(OptimizePanelUtil.FLATE_COMPRESSION);
		compressionMono.add(OptimizePanelUtil.JBIG2BW);
		
		Vector compressionGray = new Vector();
		compressionGray.add(OptimizePanelUtil.DO_NOT_CHANGE);
		compressionGray.add(OptimizePanelUtil.JPEG_COMPRESSION);
		compressionGray.add(OptimizePanelUtil.JPEG2000_COMPRESSION);
		compressionGray.add(OptimizePanelUtil.FLATE_COMPRESSION);
		compressionGray.add(OptimizePanelUtil.JBIG2BW);
		
		Vector colorSpaceGray = new Vector();
		colorSpaceGray.add(OptimizePanelUtil.DO_NOT_CHANGE);
		colorSpaceGray.add(OptimizePanelUtil.BW_COLORSPACE);
				
		Vector compressionColor = (Vector) compressionGray.clone();
		
		Vector colorSpaceColor = new Vector();
		colorSpaceColor.add(OptimizePanelUtil.DO_NOT_CHANGE);
		colorSpaceColor.add(OptimizePanelUtil.GRAY_COLORSPACE);
		colorSpaceColor.add(OptimizePanelUtil.BW_COLORSPACE);
		
		m_ColorController = new OptimizeImagePanelController(m_Dialog, m_Dialog.getJpColorImages(), compressionColor, colorSpaceColor);
		m_GrayController = new OptimizeImagePanelController(m_Dialog, m_Dialog.getJpGrayImages(), compressionGray, colorSpaceGray);
		m_MonoController = new OptimizeImagePanelController(m_Dialog, m_Dialog.getJpMonoImages(), compressionMono, null);
	}

	private void loadPreferences()
	{
		m_Dialog.getJcbFlateUncompressedStreams().setSelected(m_Options.isFlateUncompressedStreams());
		m_Dialog.getJcbFlattenAnnotations().setSelected(m_Options.isFlattenAnnotations());
		m_Dialog.getJcbFlattenFormFields().setSelected(m_Options.isFlattenFormFields());
		m_Dialog.getJcbMergeDuplicateFonts().setSelected(m_Options.isMergeDuplicateFonts());
		m_Dialog.getJcbMergeDuplicateImages().setSelected(m_Options.isMergeDuplicateImages());
		m_Dialog.getJcbRemoveAltImages().setSelected(m_Options.isDiscardAltImages());
		m_Dialog.getJcbRemoveAnnotations().setSelected(m_Options.isDiscardAnnotations());
		m_Dialog.getJcbRemoveBookmarks().setSelected(m_Options.isDiscardBookmarks());
		m_Dialog.getJcbRemoveDocumentInfo().setSelected(m_Options.isDiscardDocumentInfo());
		m_Dialog.getJcbRemoveFileAttachments().setSelected(m_Options.isDiscardFileAttachments());
		m_Dialog.getJcbRemoveFormFields().setSelected(m_Options.isDiscardFormFields());
		m_Dialog.getJcbRemoveJSActions().setSelected(m_Options.isDiscardJSActions());
		m_Dialog.getJcbRemovePageThumbnails().setSelected(m_Options.isDiscardPageThumbnails());
		m_Dialog.getJcbRemoveXMPData().setSelected(m_Options.isDiscardXMPData());
		m_Dialog.getJcbClearSignatures().setSelected(m_Options.isClearSignatures());
		m_Dialog.getJcbRemoveUnusedObjects().setSelected(m_Options.isDiscardUnusedResources());
		m_Dialog.getJcbRemoveLinks().setSelected(m_Options.isDiscardLinks());
		m_Dialog.getJcbLinearize().setSelected(m_Options.isLinearize());
		m_Dialog.getJcbObjectStreams().setSelected(m_Options.isCompressObjectsIntoStreams());
		
		CompositeImageHandler imageHandler = (CompositeImageHandler) m_Options.getImageHandler();
		if (imageHandler != null)
		{
			List imageConverts = imageHandler.getImageHandlers();
			for (int i = 0; i < imageConverts.size(); i++)
			{
				if (imageConverts.get(i) instanceof SampleImageHandler)
				{
					SampleImageHandler ic = ((SampleImageHandler) imageConverts.get(i));
					if (ic.getFilter() == SampleImageHandler.COLOR_FILTER)
					{
						m_ColorController.setImageConvert(ic);
					}
					else if (ic.getFilter() == SampleImageHandler.GRAY_FILTER)
					{
						m_GrayController.setImageConvert(ic);
					}
					else if (ic.getFilter() == SampleImageHandler.MONO_FILTER)
					{
						m_MonoController.setImageConvert(ic);
					}
				}
			}
		}
	}

	private void addListeners()
	{
		m_Dialog.getjbCancel().addActionListener(this);
		m_Dialog.getjbOk().addActionListener(this);

		m_MonoController.addListeners();
		m_ColorController.addListeners();
		m_GrayController.addListeners();

		m_Dialog.getJcbRemoveAnnotations().addItemListener(this);
		m_Dialog.getJcbRemoveFormFields().addItemListener(this);
	}

	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource().equals(m_Dialog.getjbCancel()))
		{
			m_Dialog.dispose();
		}
		else if (e.getSource().equals(m_Dialog.getjbOk()))
		{
			if (validateDPI())
			{
				getOptInfo();
				m_Dialog.dispose();
			}
		}
	}

	private OptSettings getOptInfo()
	{
		m_Options.setImageHandler(null);

		// images
		Vector imageHandlers = new Vector();
		addImageHandler(imageHandlers, m_ColorController.getImageConvert(SampleImageHandler.COLOR_FILTER));
		addImageHandler(imageHandlers, m_GrayController.getImageConvert(SampleImageHandler.GRAY_FILTER));
		addImageHandler(imageHandlers, m_MonoController.getImageConvert(SampleImageHandler.MONO_FILTER));
		
		if (imageHandlers.size() > 0)
		{
			m_Options.setImageHandler(new CompositeImageHandler(imageHandlers));
		}
		
		// remove
		m_Options.setDiscardAltImages(m_Dialog.getJcbRemoveAltImages().isSelected());
		m_Options.setDiscardAnnotations(m_Dialog.getJcbRemoveAnnotations().isSelected());
		m_Options.setDiscardBookmarks(m_Dialog.getJcbRemoveBookmarks().isSelected());
		m_Options.setDiscardDocumentInfo(m_Dialog.getJcbRemoveDocumentInfo().isSelected());
		m_Options.setDiscardFileAttachments(m_Dialog.getJcbRemoveFileAttachments().isSelected());
		m_Options.setDiscardFormFields(m_Dialog.getJcbRemoveFormFields().isSelected());
		m_Options.setDiscardJSActions(m_Dialog.getJcbRemoveJSActions().isSelected());
		m_Options.setDiscardPageThumbnails(m_Dialog.getJcbRemovePageThumbnails().isSelected());
		m_Options.setDiscardXMPData(m_Dialog.getJcbRemoveXMPData().isSelected());
		m_Options.setClearSignatures(m_Dialog.getJcbClearSignatures().isSelected());
		m_Options.setDiscardUnusedResources(m_Dialog.getJcbRemoveUnusedObjects().isSelected());
		m_Options.setDiscardLinks(m_Dialog.getJcbRemoveLinks().isSelected());
		
		// convert
		m_Options.setFlattenAnnotations(m_Dialog.getJcbFlattenAnnotations().isSelected());
		m_Options.setFlattenFormFields(m_Dialog.getJcbFlattenFormFields().isSelected());
		m_Options.setFlateUncompressedStreams(m_Dialog.getJcbFlateUncompressedStreams().isSelected());
		m_Options.setMergeDuplicateFonts(m_Dialog.getJcbMergeDuplicateFonts().isSelected());
		m_Options.setMergeDuplicateImages(m_Dialog.getJcbMergeDuplicateImages().isSelected());
		m_Options.setCompressObjectsIntoStreams(m_Dialog.getJcbObjectStreams().isSelected());
		
		//linearize
		m_Options.setLinearize(m_Dialog.getJcbLinearize().isSelected());
		
		return m_Options;
	}
	
	private void addImageHandler(Vector v, ImageHandler handler)
	{
		if (handler != null)
		{
			v.add(handler);
		}
	}

	public void itemStateChanged(ItemEvent e)
	{
		if (e.getSource().equals(m_Dialog.getJcbRemoveAnnotations()))
		{
			toggleCheckBox(m_Dialog.getJcbRemoveAnnotations(), m_Dialog.getJcbFlattenAnnotations());
		}
		else if (e.getSource().equals(m_Dialog.getJcbRemoveFormFields()))
		{
			toggleCheckBox(m_Dialog.getJcbRemoveFormFields(), m_Dialog.getJcbFlattenFormFields());
		}
	}

	private void toggleCheckBox(JCheckBox source, JCheckBox affected)
	{
		boolean checked = source.isSelected();
		if (checked)
		{
			affected.setSelected(false);
		}
		affected.setEnabled(!checked);
	}

	private boolean validateDPI()
	{
		return m_ColorController.validateDPI() && m_GrayController.validateDPI() && m_MonoController.validateDPI();
	}
}
