package jPDFOptimizerSamples;

import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.Border;

public class OptimizationOptionsDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	private JButton jbOK;
	private JButton jbCancel;

	private JPanel jPanel;
	private JTabbedPane jtpTabbedPane;
	private JPanel jpChangeContent;

	private JPanel jpImages;
	private OptimizeImagePanel jpColorImages;
	private OptimizeImagePanel jpGrayImages;
	private OptimizeImagePanel jpMonoImages;

	private JPanel jpConvert;
	private JCheckBox jcbFlattenAnnotations;
	private JCheckBox jcbFlattenFormFields;
	private JCheckBox jcbFlateUncompressedStreams;
	private JCheckBox jcbMergeDuplicateImages;
	private JCheckBox jcbMergeDuplicateFonts;
	private JCheckBox jcbClearSignatures;
	private JCheckBox jcbObjectStreams;
	
	private JPanel jpLinearize;
	private JCheckBox jcbLinearize;

	private JPanel jpRemove;
	private JCheckBox jcbRemoveAltImages;
	private JCheckBox jcbRemovePageThumbnails;
	private JCheckBox jcbRemoveJSActions;
	private JCheckBox jcbRemoveXMPData;
	private JCheckBox jcbRemoveDocumentInfo;
	private JCheckBox jcbRemoveBookmarks;
	private JCheckBox jcbRemoveAnnotations;
	private JCheckBox jcbRemoveFormFields;
	private JCheckBox jcbRemoveFileAttachments;
	private JCheckBox jcbRemoveObjects;
	private JCheckBox jcbRemoveLinks;
		
	private static final Border EMPTY_BORDER = BorderFactory.createEmptyBorder(10, 10, 10, 10);
	private static final Font CB_FONT = new Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(new JLabel().getFont().getSize(), (int)SampleUtil.getScaledFont(new JLabel().getFont().getSize(), 10)));
	
	public OptimizationOptionsDialog(Frame frame)
	{
		super(frame);
		initialize();
	}

	public void initialize()
	{
		this.setModal(true);
		this.setContentPane(getJPanel());
		this.setTitle("Optimization Options");
		this.pack();
	}

	private JPanel getJPanel()
	{
		if (jPanel == null)
		{
			jPanel = new JPanel();
			BoxLayout layout = new BoxLayout(jPanel, BoxLayout.Y_AXIS);
			jPanel.setLayout(layout);
			jPanel.setBorder(EMPTY_BORDER);
			
			jPanel.add(getTabbedPane());
			
			JPanel jpButtons = new JPanel();
			jpButtons.setLayout(new GridLayout(1, 4, 20, 0));
			jpButtons.setBorder(EMPTY_BORDER);
			jpButtons.add(new JLabel(" "));
			jpButtons.add(getjbOk());
			jpButtons.add(getjbCancel());
			jpButtons.add(new JLabel(" "));
			jPanel.add(jpButtons);
		}
		return jPanel;
	}
	
	private JTabbedPane getTabbedPane()
	{
		if (jtpTabbedPane == null)
		{
			jtpTabbedPane = new JTabbedPane();
			jtpTabbedPane.addTab("Image Processing", getJpImages());
			jtpTabbedPane.addTab("Advanced", getJpContent());
		}
		return jtpTabbedPane;
	}

	private JPanel getJpContent()
	{
		if (jpChangeContent == null)
		{
			jpChangeContent = new JPanel();
			jpChangeContent.setBorder(EMPTY_BORDER);			
			jpChangeContent.setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.BOTH;
			c.weightx = 1;
			c.gridx = 0;
			c.gridy = 0;
			jpChangeContent.add(getJpRemove(), c);
			c.gridx = 1;
			jpChangeContent.add(getJpConvert(), c);
			c.gridx = 0;
			c.gridy = 1;
			c.gridwidth = 2;
			jpChangeContent.add(getJpLinearize(), c);
			// add an empty component that takes up the remaining space
			c.gridy = 2;
			c.weighty = 1;
			jpChangeContent.add(new JLabel(), c);
			
		}
		return jpChangeContent;
	}

	public JPanel getJpImages()
	{
		if (jpImages == null)
		{
			jpImages = new JPanel();
			jpImages.setBorder(EMPTY_BORDER);
			BoxLayout layout = new BoxLayout(jpImages, BoxLayout.Y_AXIS);
			jpImages.setLayout(layout);
			jpImages.add(getJpColorImages());
			jpImages.add(getJpGrayImages());
			jpImages.add(getJpMonoImages());
		}
		return jpImages;
	}

	public OptimizeImagePanel getJpColorImages()
	{
		if (jpColorImages == null)
		{
			jpColorImages = new OptimizeImagePanel("Color Images", true);
		}
		return jpColorImages;
	}

	public OptimizeImagePanel getJpGrayImages()
	{
		if (jpGrayImages == null)
		{
			jpGrayImages = new OptimizeImagePanel("Grayscale Images", true);
		}
		return jpGrayImages;
	}

	public OptimizeImagePanel getJpMonoImages()
	{
		if (jpMonoImages == null)
		{
			jpMonoImages = new OptimizeImagePanel("Black and White Images", false);
		}
		return jpMonoImages;
	}

	private JPanel getJpRemove()
	{
		if (jpRemove == null)
		{
			jpRemove = new JPanel();
			jpRemove.setBorder(BorderFactory.createTitledBorder("Object Removal"));
			BoxLayout layout = new BoxLayout(jpRemove, BoxLayout.X_AXIS);
			jpRemove.setLayout(layout);
			
			JPanel jpLeft = new JPanel();
			BoxLayout layoutL = new BoxLayout(jpLeft, BoxLayout.Y_AXIS);
			jpLeft.setLayout(layoutL);
			
			JPanel jpRight = new JPanel();
			BoxLayout layoutR = new BoxLayout(jpRight, BoxLayout.Y_AXIS);
			jpRight.setLayout(layoutR);
			
			jpLeft.add(getJcbRemoveAltImages());
			jpRight.add(getJcbRemoveAnnotations());
			jpLeft.add(getJcbRemoveBookmarks());
			jpRight.add(getJcbRemoveDocumentInfo());
			jpLeft.add(getJcbRemoveFileAttachments());
			jpRight.add(getJcbRemoveFormFields());
			jpLeft.add(getJcbRemoveJSActions());
			jpRight.add(getJcbRemovePageThumbnails());
			jpLeft.add(getJcbRemoveXMPData());
			jpRight.add(getJcbRemoveUnusedObjects());
			jpLeft.add(getJcbRemoveLinks());
			jpRight.add(Box.createVerticalGlue());
			jpLeft.add(Box.createVerticalGlue());
			
			jpRemove.add(jpLeft);
			jpRemove.add(Box.createHorizontalStrut(20));
			jpRemove.add(Box.createHorizontalGlue());
			jpRemove.add(jpRight);
		}
		return jpRemove;
	}

	public JCheckBox getJcbRemoveAltImages()
	{
		if (jcbRemoveAltImages == null)
		{
			jcbRemoveAltImages = new JCheckBox("Alternate Images");
			jcbRemoveAltImages.setFont(CB_FONT);
		}
		return jcbRemoveAltImages;
	}

	public JCheckBox getJcbRemoveAnnotations()
	{
		if (jcbRemoveAnnotations == null)
		{
			jcbRemoveAnnotations = new JCheckBox("Comments");
			jcbRemoveAnnotations.setFont(CB_FONT);
		}
		return jcbRemoveAnnotations;
	}

	public JCheckBox getJcbRemoveBookmarks()
	{
		if (jcbRemoveBookmarks == null)
		{
			jcbRemoveBookmarks = new JCheckBox("Bookmarks");
			jcbRemoveBookmarks.setFont(CB_FONT);
		}
		return jcbRemoveBookmarks;
	}

	public JCheckBox getJcbRemoveDocumentInfo()
	{
		if (jcbRemoveDocumentInfo == null)
		{
			jcbRemoveDocumentInfo = new JCheckBox("Document Info");
			jcbRemoveDocumentInfo.setFont(CB_FONT);
		}
		return jcbRemoveDocumentInfo;
	}

	public JCheckBox getJcbRemoveFileAttachments()
	{
		if (jcbRemoveFileAttachments == null)
		{
			jcbRemoveFileAttachments = new JCheckBox("Attachments");
			jcbRemoveFileAttachments.setFont(CB_FONT);
		}
		return jcbRemoveFileAttachments;
	}

	public JCheckBox getJcbRemoveFormFields()
	{
		if (jcbRemoveFormFields == null)
		{
			jcbRemoveFormFields = new JCheckBox("Form Fields");
			jcbRemoveFormFields.setFont(CB_FONT);
		}
		return jcbRemoveFormFields;
	}

	public JCheckBox getJcbRemoveJSActions()
	{
		if (jcbRemoveJSActions == null)
		{
			jcbRemoveJSActions = new JCheckBox("JavaScript Actions");
			jcbRemoveJSActions.setFont(CB_FONT);
		}
		return jcbRemoveJSActions;
	}

	public JCheckBox getJcbRemovePageThumbnails()
	{
		if (jcbRemovePageThumbnails == null)
		{
			jcbRemovePageThumbnails = new JCheckBox("Embedded Thumbnails");
			jcbRemovePageThumbnails.setFont(CB_FONT);
		}
		return jcbRemovePageThumbnails;
	}

	public JCheckBox getJcbRemoveXMPData()
	{
		if (jcbRemoveXMPData == null)
		{
			jcbRemoveXMPData = new JCheckBox("Advanced Metadata");
			jcbRemoveXMPData.setFont(CB_FONT);
		}
		return jcbRemoveXMPData;
	}

	public JCheckBox getJcbRemoveUnusedObjects()
	{
		if (jcbRemoveObjects == null)
		{
			jcbRemoveObjects = new JCheckBox("Unused Resources");
			jcbRemoveObjects.setFont(CB_FONT);
		}
		return jcbRemoveObjects;
	}
	
	public JCheckBox getJcbRemoveLinks()
	{
		if (jcbRemoveLinks == null)
		{
			jcbRemoveLinks = new JCheckBox("Links");
			jcbRemoveLinks.setFont(CB_FONT);
		}
		return jcbRemoveLinks;
	}

	private JPanel getJpConvert()
	{
		if (jpConvert == null)
		{
			jpConvert = new JPanel();
			BoxLayout layout = new BoxLayout(jpConvert, BoxLayout.Y_AXIS);
			jpConvert.setLayout(layout);
			jpConvert.setBorder(BorderFactory.createTitledBorder("Other"));
			jpConvert.add(getJcbFlattenAnnotations());
			jpConvert.add(getJcbMergeDuplicateImages());
			jpConvert.add(getJcbFlattenFormFields());
			jpConvert.add(getJcbMergeDuplicateFonts());
			jpConvert.add(getJcbFlateUncompressedStreams());
			jpConvert.add(getJcbClearSignatures());
			jpConvert.add(getJcbObjectStreams());
			jpConvert.add(Box.createVerticalGlue());
		}
		return jpConvert;
	}
	
	private JPanel getJpLinearize()
	{
		if (jpLinearize == null)
		{
			jpConvert = new JPanel(new FlowLayout(FlowLayout.LEADING));
			jpConvert.setBorder(BorderFactory.createTitledBorder("Fast Web View"));
			jpConvert.add(getJcbLinearize());
		}
		return jpConvert;
	}

	public JCheckBox getJcbFlattenAnnotations()
	{
		if (jcbFlattenAnnotations == null)
		{
			jcbFlattenAnnotations = new JCheckBox("Flatten All Comments");
			jcbFlattenAnnotations.setFont(CB_FONT);
		}
		return jcbFlattenAnnotations;
	}
	
	public JCheckBox getJcbLinearize()
	{
		if (jcbLinearize == null)
		{
			jcbLinearize = new JCheckBox("Convert to fast web view");
			jcbLinearize.setFont(CB_FONT);
		}
		return jcbLinearize;
	}

	public JCheckBox getJcbMergeDuplicateImages()
	{
		if (jcbMergeDuplicateImages == null)
		{
			jcbMergeDuplicateImages = new JCheckBox("Merge Duplicate Images");
			jcbMergeDuplicateImages.setFont(CB_FONT);
		}
		return jcbMergeDuplicateImages;
	}

	public JCheckBox getJcbFlattenFormFields()
	{
		if (jcbFlattenFormFields == null)
		{
			jcbFlattenFormFields = new JCheckBox("Flatten All Fields");
			jcbFlattenFormFields.setFont(CB_FONT);
		}
		return jcbFlattenFormFields;
	}

	public JCheckBox getJcbMergeDuplicateFonts()
	{
		if (jcbMergeDuplicateFonts == null)
		{
			jcbMergeDuplicateFonts = new JCheckBox("Merge Duplicate Fonts");
			jcbMergeDuplicateFonts.setFont(CB_FONT);
		}
		return jcbMergeDuplicateFonts;
	}

	public JCheckBox getJcbFlateUncompressedStreams()
	{
		if (jcbFlateUncompressedStreams == null)
		{
			jcbFlateUncompressedStreams = new JCheckBox("Flate Uncompressed Streams");
			jcbFlateUncompressedStreams.setFont(CB_FONT);
		}
		return jcbFlateUncompressedStreams;
	}
	
	public JCheckBox getJcbObjectStreams()
	{
		if (jcbObjectStreams == null)
		{
			jcbObjectStreams = new JCheckBox("Compress document objects");
			jcbObjectStreams.setFont(CB_FONT);
		}
		return jcbObjectStreams;
	}

	public JCheckBox getJcbClearSignatures()
	{
		if (jcbClearSignatures == null)
		{
			jcbClearSignatures = new JCheckBox("Clear Signatures");
			jcbClearSignatures.setFont(CB_FONT);
		}
		return jcbClearSignatures;
	}

	public JButton getjbOk()
	{
		if (jbOK == null)
		{
			jbOK = new JButton();
			jbOK.setFont(CB_FONT);
			jbOK.setText("Ok");
		}
		return jbOK;
	}

	public JButton getjbCancel()
	{
		if (jbCancel == null)
		{
			jbCancel = new JButton();
			jbCancel.setFont(CB_FONT);
			jbCancel.setText("Cancel");
		}
		return jbCancel;
	}

} // @jve:decl-index=0:visual-constraint="10,10"
