package jPDFOptimizerSamples;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.plaf.basic.BasicButtonUI;

public class OptimizeImagePanel extends JPanel
{
	private JLabel jlColorSpace;
	private JComboBox jcbColorSpace;
	private JLabel jlCompression;
	private JComboBox jcbCompression;
	private JLabel jlQuality;
	private JSpinner jsQuality;
	private HelpButton qhQuality;
	private JLabel jlDpi;
	private JComboBox jcbDpi;
	private HelpButton qhDpi;
	private JLabel jlOtherDpi;
	private IntegerRangeTextField jtfOtherDpi;	

	private static final String OTHER_DPI_FAIL_MESSAGE = "Please enter a DPI value between 50 - 1200";
	private static final Font LABEL_FONT = new Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(new JLabel().getFont().getSize(), 10));

	public OptimizeImagePanel(String title, boolean includeColorSpace)
	{
		super();
		initialize(title, includeColorSpace);
	}
		
	private void initialize(String title, boolean includeColorSpace)
	{
		setBorder(BorderFactory.createTitledBorder(title));
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1;
		c.weightx = 1;
		// setting an x inset cuts stuff off in linux, the best way to create x space seems to be with a blank label
		c.insets = new Insets(2, 2, 2, 2);
		c.ipadx = 5;
		c.anchor = GridBagConstraints.EAST;
		
		// help buttons are very wide on mac, so set a different ui
		getQhDpi().setUI(new BasicButtonUI());
		getQhQuality().setUI(new BasicButtonUI());
		
		add(getJlCompression(), c);
		c.gridwidth = 2;
		add(getJcbCompression(), c);
		c.gridwidth = 1;
		add(new JLabel(" "), c);
		
		if (includeColorSpace)
		{
			add(getJlColorSpace(), c);
			c.gridwidth = 2;
			add(getJcbColorSpace(), c);
			c.gridwidth = 1;
		}
		else
		{
			add(getJlDpi(), c);
			add(getJcbDpi(), c);
			c.fill = GridBagConstraints.VERTICAL;
			add(getQhDpi(), c);
			c.fill = GridBagConstraints.BOTH;
		}
		
		c.gridy = 1;

		if (includeColorSpace)
		{
			add(getJlQuality(), c);
			add(getJsQuality(), c);
			c.fill = GridBagConstraints.VERTICAL;
			add(getQhQuality(), c);
			c.fill = GridBagConstraints.BOTH;
			add(new JLabel(" "), c);
			
			add(getJlDpi(), c);
			add(getJcbDpi(), c);
			c.fill = GridBagConstraints.VERTICAL;
			add(getQhDpi(), c);
			c.fill = GridBagConstraints.BOTH;
			c.gridy = 2;
		}
		
		// take up some space on the bottom left
		add(new JLabel(), c);
		add(new JLabel(), c);
		add(new JLabel(), c);
		add(new JLabel(" "), c);
		
		add(getJlOtherDpi(), c);
		JPanel placeHolder = new JPanel(new GridLayout(1, 1));
		placeHolder.add(getJtfOtherDpi());
		add(placeHolder, c);
	}
	
	public JLabel getJlCompression()
	{
		if (jlCompression == null)
		{
			jlCompression = new JLabel("Compression: ", SwingConstants.RIGHT);
			jlCompression.setFont(LABEL_FONT);
		}
		return jlCompression;
	}
	
	public JComboBox getJcbCompression()
	{
		if (jcbCompression == null)
		{
			jcbCompression = new JComboBox();
		}
		return jcbCompression;
	}

	public JLabel getJlDpi()
	{
		if (jlDpi == null)
		{
			jlDpi = new JLabel("DPI Resolution: ", SwingConstants.RIGHT);
			jlDpi.setFont(LABEL_FONT);
		}
		return jlDpi;
	}

	public JComboBox getJcbDpi()
	{
		if (jcbDpi == null)
		{
			jcbDpi = new JComboBox();
		}
		return jcbDpi;
	}

	public HelpButton getQhDpi()
	{
		if (qhDpi == null)
		{
			qhDpi = new HelpButton();
			qhDpi.setHelpText("Images with resolution greater than the selected resolution will have the DPI converted.  DPI below 150 is not recommended as this will severely degrade images.");
		}
		return qhDpi;
	}

	public JLabel getJlOtherDpi()
	{
		if (jlOtherDpi == null)
		{
			jlOtherDpi = new JLabel("Other: ", SwingConstants.RIGHT);
			jlOtherDpi.setFont(LABEL_FONT);
		}
		return jlOtherDpi;
	}
	
	public IntegerRangeTextField getJtfOtherDpi()
	{
		if (jtfOtherDpi == null)
		{      
			jtfOtherDpi = new IntegerRangeTextField(300, 50, 1200, 4, OTHER_DPI_FAIL_MESSAGE);
		}
		return jtfOtherDpi;
	}

	public JLabel getJlQuality()
	{
		if (jlQuality == null)
		{
			jlQuality = new JLabel("Quality: ", SwingConstants.RIGHT);
			jlQuality.setFont(LABEL_FONT);
		}
		return jlQuality;
	}

	public JSpinner getJsQuality()
	{
		if (jsQuality == null)
		{
			jsQuality = new JSpinner(new SpinnerNumberModel(8, 1, 10, 1));
		}
		return jsQuality;
	}

	public HelpButton getQhQuality()
	{
		if (qhQuality == null)
		{
			qhQuality = new HelpButton();
			qhQuality.setHelpText("This is the quality of JPEG compression. It varies from 1 (very high compression, very low quality), to 10 (low compression, very high quality) based on the quality parameter. Some guidelines for the quality value: 8 high quality, 5  medium quality, 2 low quality.");
		}
		return qhQuality;
	}

	public JLabel getJlColorSpace()
	{
		if (jlColorSpace == null)
		{
			jlColorSpace = new JLabel("Color Space: ", SwingConstants.RIGHT);
			jlColorSpace.setFont(LABEL_FONT);
		}
		return jlColorSpace;
	}

	public JComboBox getJcbColorSpace()
	{
		if (jcbColorSpace == null)
		{
			jcbColorSpace = new JComboBox();
		}
		return jcbColorSpace;
	}
	
}
