package jPDFOptimizerSamples;

import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import com.qoppa.pdfOptimizer.ImageHandler;
import com.qoppa.pdfOptimizer.ImageOutput;

public class OptimizeImagePanelController implements ActionListener
{
	private OptimizeImagePanel m_Panel;
	private Vector m_CompressionOptions;
	private Vector m_ColorSpaceOptions;

	private boolean m_Validate = true;
	private OptimizationOptionsDialog m_ParentDialog;

	// Have to keep these separate since both color and compression can have the "NoChange" key and
	// validators will be different
	private static HashMap colorValidators; // String, Validator
	private static HashMap compressionValidators; // String, Validator
	private static String COLOR_UPDATED = "Image color space updated for compatibility";
	private static String COMPRESSION_UPDATED = "Image compression updated for compatibility";

	static
	{
		colorValidators = new HashMap();

		Vector jpeg = new Vector(Arrays.asList(new String[] { OptimizePanelUtil.DO_NOT_CHANGE, OptimizePanelUtil.GRAY_COLORSPACE }));
		colorValidators.put(OptimizePanelUtil.JPEG_COMPRESSION, new Validator(jpeg, OptimizePanelUtil.DO_NOT_CHANGE));

		Vector jpeg2000 = new Vector(Arrays.asList(new String[] { OptimizePanelUtil.DO_NOT_CHANGE, OptimizePanelUtil.GRAY_COLORSPACE }));
		colorValidators.put(OptimizePanelUtil.JPEG2000_COMPRESSION, new Validator(jpeg2000, OptimizePanelUtil.DO_NOT_CHANGE));

		Vector jbig = new Vector(Arrays.asList(new String[] { OptimizePanelUtil.BW_COLORSPACE }));
		colorValidators.put(OptimizePanelUtil.JBIG2BW, new Validator(jbig, OptimizePanelUtil.BW_COLORSPACE));

		// flate compression and no change (compression) do not require validation since all options
		// are applicable

		compressionValidators = new HashMap();

		Vector noChangeColor = new Vector(Arrays.asList(new String[] { OptimizePanelUtil.DO_NOT_CHANGE, OptimizePanelUtil.JPEG_COMPRESSION, OptimizePanelUtil.JPEG2000_COMPRESSION, OptimizePanelUtil.FLATE_COMPRESSION }));
		compressionValidators.put(OptimizePanelUtil.DO_NOT_CHANGE, new Validator(noChangeColor, OptimizePanelUtil.DO_NOT_CHANGE));

		Vector gray = new Vector(Arrays.asList(new String[] { OptimizePanelUtil.DO_NOT_CHANGE, OptimizePanelUtil.JPEG_COMPRESSION, OptimizePanelUtil.JPEG2000_COMPRESSION, OptimizePanelUtil.FLATE_COMPRESSION }));
		compressionValidators.put(OptimizePanelUtil.GRAY_COLORSPACE, new Validator(gray, OptimizePanelUtil.FLATE_COMPRESSION));

		Vector bw = new Vector(Arrays.asList(new String[] { OptimizePanelUtil.FLATE_COMPRESSION, OptimizePanelUtil.JBIG2BW }));
		compressionValidators.put(OptimizePanelUtil.BW_COLORSPACE, new Validator(bw, OptimizePanelUtil.JBIG2BW));

	}
	
	public OptimizeImagePanelController(OptimizationOptionsDialog dialog, OptimizeImagePanel panel, Vector compressionOptions, Vector colorSpaceOptions)
	{
		m_ParentDialog = dialog;
		m_Panel = panel;
		m_CompressionOptions = compressionOptions;
		m_ColorSpaceOptions = colorSpaceOptions;
		
		fillCombos();
	}

	void fillCombos()
	{
		// fill compression combo
		m_Panel.getJcbCompression().setModel(new DefaultComboBoxModel(m_CompressionOptions));
		
		// fill color space combo, could be null
		if (m_ColorSpaceOptions != null)
		{
			m_Panel.getJcbColorSpace().setModel(new DefaultComboBoxModel(m_ColorSpaceOptions));
		}
		else
		{
			m_Panel.getJcbColorSpace().setVisible(false);
			m_Panel.getJlColorSpace().setVisible(false);
		}
		
		m_Panel.getJcbDpi().setModel(new DefaultComboBoxModel(OptimizePanelUtil.DPI_OPTIONS));
	}
	
	void addListeners()
	{			
		m_Panel.getJcbCompression().addActionListener(this);
		m_Panel.getJcbColorSpace().addActionListener(this);
		m_Panel.getJcbDpi().addActionListener(this);
	}
	
	private void toggleQuality()
	{
		Object selected = m_Panel.getJcbCompression().getSelectedItem();
		boolean jpeg = selected.equals(OptimizePanelUtil.JPEG_COMPRESSION) || selected.equals(OptimizePanelUtil.JPEG2000_COMPRESSION);
		m_Panel.getJsQuality().setEnabled(jpeg);
		m_Panel.getJlQuality().setEnabled(jpeg);
	}

	private void toggleDPI()
	{
		Object selected = m_Panel.getJcbDpi().getSelectedItem();
		boolean other = selected.equals(OptimizePanelUtil.OTHER);
		m_Panel.getJlOtherDpi().setVisible(other);
		m_Panel.getJtfOtherDpi().setVisible(other);
	}

	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource().equals(m_Panel.getJcbCompression()))
		{
			toggleQuality();
			validateColor();
		}
		else if (e.getSource().equals(m_Panel.getJcbDpi()))
		{
			toggleDPI();
		}
		else if (e.getSource().equals(m_Panel.getJcbColorSpace()))
		{
			validateCompression();
		}
	}

	private void validateColor()
	{
		validate((Validator) colorValidators.get((String) m_Panel.getJcbCompression().getSelectedItem()), m_Panel.getJcbColorSpace(), COLOR_UPDATED);
	}

	private void validateCompression()
	{
		validate((Validator) compressionValidators.get((String) m_Panel.getJcbColorSpace().getSelectedItem()), m_Panel.getJcbCompression(), COMPRESSION_UPDATED);
	}

	private void validate(Validator validator, JComboBox combo, final String message)
	{
		if (m_Validate)
		{
			if (validator != null && !validator.isValid((String) combo.getSelectedItem()))
			{
				combo.setSelectedItem(validator.getDefaultValue());
				// ^still need to update for when we read settings & do the conversion
				// only tell the user if the combo is visible (in b&w the color combo is not visible)
				if (combo.isVisible())
				{
					SwingUtilities.invokeLater(new Runnable() {
						public void run()
						{
							Rectangle rect = m_ParentDialog.getBounds();
							rect.x = 0;
							rect.y = 0;
							showToast(m_Panel.getRootPane(), message, rect);
						}
					});
				}
			}
		}
	}
	
	private void showToast(final JRootPane rootPane, String message, Rectangle parentRect)
	{
		final JPanel toastPanel = new JPanel();
		toastPanel.setBackground(new Color(48, 48, 48));
		toastPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		JLabel label = new JLabel();
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		label.setForeground(Color.WHITE);
		label.setText(message);
		toastPanel.add(label);
		toastPanel.setSize(toastPanel.getPreferredSize());
		
		toastPanel.setLocation((parentRect.x + parentRect.width - toastPanel.getWidth()) / 2, (parentRect.y + parentRect.height - toastPanel.getHeight()) / 2);
		toastPanel.setVisible(false);
		toastPanel.addComponentListener(new ComponentAdapter() {
			public void componentShown(ComponentEvent e)
			{
				Timer timer = new Timer(2000, new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						Container parent = toastPanel.getParent();
						if (parent != null)
						{
							parent.remove(toastPanel);
							rootPane.repaint();
						}
					}
				});
				timer.setRepeats(false);
				timer.start();
			}
		});
		rootPane.getLayeredPane().add(toastPanel, JLayeredPane.POPUP_LAYER);
		toastPanel.setVisible(true);
	}
	
	public ImageHandler getImageConvert(int filter)
	{
		// this will be null or an Integer
		Integer maxDpi = getMaxDpi();
		
		// this will always have a value, may be ImageOutput.CS_RETAIN
		int colorSpace = getColorSpace(m_Panel.getJcbColorSpace().getSelectedItem());
		// this will always have a value, may be ImageOutput.CO_RETAIN
		int compression = getCompression(m_Panel.getJcbCompression().getSelectedItem());
		Double quality = null;
		if (compression != ImageOutput.CO_RETAIN && (compression == ImageOutput.CO_JPEG || compression == ImageOutput.CO_JPEG2000))
		{
			quality = getQuality();
		}

		// only return an image convert if we're changing something
		if (maxDpi != null || colorSpace != ImageOutput.CS_RETAIN || compression != ImageOutput.CO_RETAIN)
		{
			return new SampleImageHandler(filter, maxDpi, new Integer(colorSpace), new Integer(compression), quality);
		}
		else
		{
			return null;
		}
	}
	
	private Double getQuality()
	{
		// divide by 10 to get a float value
		return new Double(Integer.valueOf(m_Panel.getJsQuality().getValue().toString()).intValue() / 10f);
	}
	
	private int getCompression(Object selection)
	{
		if (selection.equals(OptimizePanelUtil.FLATE_COMPRESSION))
		{
			return ImageOutput.CO_FLATE;
		}
		else if (selection.equals(OptimizePanelUtil.JBIG2BW))
		{
			return ImageOutput.CO_JBIG2;
		}
		else if (selection.equals(OptimizePanelUtil.JPEG2000_COMPRESSION))
		{
			return ImageOutput.CO_JPEG2000;
		}
		else if (selection.equals(OptimizePanelUtil.JPEG_COMPRESSION))
		{
			return ImageOutput.CO_JPEG;
		}
		else
		{
			return ImageOutput.CO_RETAIN;
		}
	}
	
	private void setCompression(SampleImageHandler ic)
	{
		int compression = ic.getCompression().intValue();	
		
		// this is called when loading an existing profile so we should not validate the combos
		m_Validate = false;
		
		if (compression == ImageOutput.CO_FLATE)
		{
			m_Panel.getJcbCompression().setSelectedItem(OptimizePanelUtil.FLATE_COMPRESSION);
		}
		else if (compression == ImageOutput.CO_JBIG2)
		{
			m_Panel.getJcbCompression().setSelectedItem(OptimizePanelUtil.JBIG2BW);
		}
		else if (compression == ImageOutput.CO_JPEG)
		{
			m_Panel.getJcbCompression().setSelectedItem(OptimizePanelUtil.JPEG_COMPRESSION);
			enableQuality(ic.getQuality().doubleValue());
		}
		else if (compression == ImageOutput.CO_JPEG2000)
		{
			m_Panel.getJcbCompression().setSelectedItem(OptimizePanelUtil.JPEG2000_COMPRESSION);
			enableQuality(ic.getQuality().doubleValue());
		}
		else
		{
			m_Panel.getJcbCompression().setSelectedItem(OptimizePanelUtil.DO_NOT_CHANGE);
			m_Panel.getJsQuality().setEnabled(false);
		}
		m_Validate = true;
	}
	
	private void enableQuality(double quality)
	{
		// set the percentage value - sometimes the casting/rounding is weird going from double to int, so i'll do it myself
		quality *= 10; // multiply by 10 to get percentage
		int intQuality = (int) quality;
		if (!(quality - (int) quality < .5))
		{
			// we should have rounded up, so add 1
			intQuality++;
		}
		m_Panel.getJsQuality().setValue(new Integer(intQuality));
	}

	private int getColorSpace(Object selection)
	{
		if (OptimizePanelUtil.RGB_COLORSPACE.equals(selection))
		{
			return ImageOutput.CS_RGB;
		}
		else if (OptimizePanelUtil.GRAY_COLORSPACE.equals(selection))
		{
			return ImageOutput.CS_GRAY;
		}
		else if (OptimizePanelUtil.CMYK_COLORSPACE.equals(selection))
		{
			return ImageOutput.CS_CMYK;
		}
		else if (OptimizePanelUtil.BW_COLORSPACE.equals(selection))
		{
			return ImageOutput.CS_BW;
		}
		else
		{
			return ImageOutput.CS_RETAIN;
		}
	}
	
	private void setColorSpace(int colorSpace)
	{
		// this is called when loading an existing profile, so we should not validate the combos
		m_Validate = false;
		if (colorSpace == ImageOutput.CS_RGB)
		{
			m_Panel.getJcbColorSpace().setSelectedItem(OptimizePanelUtil.RGB_COLORSPACE);
		}
		else if (colorSpace == ImageOutput.CS_GRAY)
		{
			m_Panel.getJcbColorSpace().setSelectedItem(OptimizePanelUtil.GRAY_COLORSPACE);
		}
		else if (colorSpace == ImageOutput.CS_CMYK)
		{
			m_Panel.getJcbColorSpace().setSelectedItem(OptimizePanelUtil.CMYK_COLORSPACE);
		}
		else if (colorSpace == ImageOutput.CS_BW)
		{
			m_Panel.getJcbColorSpace().setSelectedItem(OptimizePanelUtil.BW_COLORSPACE);
		}
		else
		{
			m_Panel.getJcbColorSpace().setSelectedItem(OptimizePanelUtil.DO_NOT_CHANGE);
		}
		m_Validate = true;
	}
	
	private void setDPI(String dpi)
	{
		// dpi will either be a number or "do not change"
		if (OptimizePanelUtil.DPI_OPTIONS.contains(dpi))
		{
			m_Panel.getJcbDpi().setSelectedItem(dpi);
		}
		else
		{
			m_Panel.getJcbDpi().setSelectedItem(OptimizePanelUtil.OTHER);
			m_Panel.getJtfOtherDpi().setText(dpi);
		}
	}

	public void setImageConvert(SampleImageHandler ic)
	{
		setColorSpace(ic.getColorSpace().intValue());
		setCompression(ic);
		Integer maxDpi = ic.getMaxDpi();
		setDPI(maxDpi == null ? OptimizePanelUtil.DO_NOT_CHANGE : maxDpi.toString());
	}
	
	public OptimizeImagePanel getPanel()
	{
		return m_Panel;
	}
	
	public void initDisable()
	{
		m_Panel.getJsQuality().setEnabled(false);
		m_Panel.getJlQuality().setEnabled(false);
		m_Panel.getJlOtherDpi().setVisible(false);
		m_Panel.getJtfOtherDpi().setVisible(false);
	}
	
	public Integer getMaxDpi()
	{
		Object selection = m_Panel.getJcbDpi().getSelectedItem();
		if (selection == null || selection.equals(OptimizePanelUtil.DO_NOT_CHANGE))
		{
			return null;
		}
		else if (selection.equals(OptimizePanelUtil.OTHER))
		{
			return Integer.valueOf(m_Panel.getJtfOtherDpi().getText());
		}
		else
		{
			return Integer.valueOf(selection.toString());
		}
	}

	// clears out all selected values
	public void resetPanel()
	{
		m_Panel.getJcbColorSpace().setSelectedItem(OptimizePanelUtil.DO_NOT_CHANGE);
		m_Panel.getJcbCompression().setSelectedItem(OptimizePanelUtil.DO_NOT_CHANGE);
		m_Panel.getJcbDpi().setSelectedItem(OptimizePanelUtil.DO_NOT_CHANGE);
		// set the default dpi
		m_Panel.getJtfOtherDpi().setText("" + OptimizePanelUtil.DEFAULT_DPI);
		// set the default quality
		m_Panel.getJsQuality().setValue(new Integer(OptimizePanelUtil.DEFAULT_QUALITY));
	}

	public boolean validateDPI()
	{
		if (m_Panel.getJtfOtherDpi().isEnabled())
		{
			return m_Panel.getJtfOtherDpi().verify();
		}
		return true;
	}

	static class Validator
	{
		Vector validValues;
		String defaultValue;

		public Validator(Vector validValues, String defaultValue)
		{
			this.validValues = validValues;
			this.defaultValue = defaultValue;
		}

		public boolean isValid(String value)
		{
			return validValues.contains(value);
		}

		public String getDefaultValue()
		{
			return defaultValue;
		}
	}
}
