package jPDFOptimizerSamples;

import java.beans.PropertyChangeEvent;
import java.io.FileOutputStream;

import com.qoppa.pdfOptimizer.ImageHandler;
import com.qoppa.pdfOptimizer.ImageInfo;
import com.qoppa.pdfOptimizer.ImageOutput;
import com.qoppa.pdfOptimizer.OptSettings;
import com.qoppa.pdfOptimizer.OptimizeResult;
import com.qoppa.pdfOptimizer.OptimizeResults;
import com.qoppa.pdfOptimizer.PDFOptimizer;


/**
 * 
 * OptimizerResults is a sample program showing
 * how to open a PDF document, optimize it and return optimizer results. 
 * The results are written to the console and to an xml file. 
 * 
 * Note that this OptimizerResults class implements ImageHandler
 * 
 * @author Qoppa Software
 *
 */
public class OptimizerResults implements ImageHandler
{
	
    public static void main (String [] args)
    {
        try
        {
            // Load a document
            PDFOptimizer pdfOptimize = new PDFOptimizer("C:\\test\\myfile.pdf", null);
            
            // Optimize and Return the Optimizer Results
            OptSettings options = new OptSettings();
            
            // Options to remove objects
            options.setDiscardAltImages(true);
            options.setDiscardAnnotations(true);
            options.setDiscardBookmarks(true);
            options.setDiscardDocumentInfo(true);
            options.setDiscardFileAttachments(true);
            options.setDiscardFormFields(true);
            options.setDiscardJSActions(true);
            options.setDiscardPageThumbnails(true);
            options.setDiscardXMPData(true);
            options.setDiscardUnusedResources(true);
            options.setDiscardLinks(true);
			
			// Other optimizing options
			options.setClearSignatures(true);
			options.setFlateUncompressedStreams(true);
			options.setMergeDuplicateFonts(true);
			options.setMergeDuplicateImages(true);
            
            // set the image handler
			// SimpleOptimizer extends ImageHandler
            options.setImageHandler(new OptimizerResults());
            
            options.setImageHandler(new OptimizerResults());
            
            // Optimize and save the file and return optimizer results
            OptimizeResults optimizeResults = pdfOptimize.optimize(options, "C:\\test\\myfile_opt.pdf");
            System.out.println("Optimize file saved!");
            
            // Write the optimization results to the console
            System.out.println("originalBytes " + optimizeResults.getOriginalBytes());
            System.out.println("newBytes " + optimizeResults.getNewBytes());
            if (optimizeResults.getResults() != null)
    		{
    			for (OptimizeResult result : optimizeResults.getResults())
    			{
    				// recursively write children
    				write(result);
    			}
            	
            }
            System.out.println("Optimize results output to the console!");
            
            
            // Write the optimization results to an xml file
            FileOutputStream optOut = new FileOutputStream("c:\\test\\myfile_opt_results.xml");
            optimizeResults.writeXml(optOut);
            optOut.flush();
            optOut.close();
            System.out.println("Optimize results saved!");
        }
        catch (Throwable t)
        {
            t.printStackTrace();
        }
    }
    
    // this is the method where you can implement any image conversion / optimization
	public ImageOutput convertImage(ImageInfo imageInfo) 
	{
		// construct an image output that by default retains the same image properties as the image input
		ImageOutput imageoutput = new ImageOutput(ImageOutput.CO_RETAIN, ImageOutput.CS_RETAIN, imageInfo.getImageWidth(), imageInfo.getImageHeight());
		
		// for gray or black and white images, use JBIG2 compressions
		if(imageInfo.isGray() || imageInfo.isMonochrome())
		{
			imageoutput.setCompression(ImageOutput.CO_JBIG2);
			imageoutput.setColorSpace(ImageOutput.CS_BW);
		}
		
		// for color images, use JPEG 2000 compression
		if(imageInfo.isColor())
		{
			imageoutput.setCompression(ImageOutput.CO_JPEG2000);
			imageoutput.setCompressionQuality(0.8f);
		}
		
		// downgrade image resolution to 200DPI if it's higher
		if (imageInfo.getDPIX() > 200 || imageInfo.getDPIY() > 200)
		{
			// Calculate new dimensions to match DPI
			float scale = Math.min(200 / imageInfo.getDPIX(), 200 / imageInfo.getDPIY());
			int newWidth = (int)(imageInfo.getImageWidth() * scale + 0.5);
			int newHeight = (int)(imageInfo.getImageHeight() * scale + 0.5);
			imageoutput.setImageHeight(newWidth);
			imageoutput.setImageWidth(newHeight);
		}
		
		// return the new image output
		return imageoutput;
	}
	
	// write progress
	public void propertyChange(PropertyChangeEvent evt)
	{
		System.out.println("IN PROGRESS " + (String)evt.getNewValue() + "...");
	}
	
	// this method write an OptimizeResult object and its children recursively
	public static void write(OptimizeResult result)
	{
    	System.out.println(result.getCount() + " " + result.getDescription());
    	if (result.getChildren() != null)
		{
			for (OptimizeResult curResult : result.getChildren())
				write(curResult);
		}
	}
}
