package jPDFOptimizerSamples;

import java.awt.HeadlessException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import com.qoppa.pdf.FontSettings;
import com.qoppa.pdfOptimizer.PDFOptimizer;

public class PDFOptimizerFrame extends JFrame
{
    private static boolean m_ShowAPI = true;

    public PDFOptimizerFrame() throws HeadlessException
    {
        super();
        initialize();
    }

    public static void  main (String [] args)
    {
		try
		{
			// Create a log file in the user's home directory
			File outFile = new File(System.getProperty("user.home") + File.separator + "jpdfoptimizer.log");
			PrintStream out = new PrintStream(new FileOutputStream(outFile));
			System.setOut(out);
			System.setErr(out);
			
			// Initial log information
			SampleUtil.libraryLogStart(PDFOptimizer.class);			
		}
		catch (Throwable t)
		{
			// Do nothing, continue without log file
		}

        // Set the look and feel
        setLookAndFeel();
        
		// When rendering, use a substitute font when the specified font cannot
		// be found, or the is an error with the embedded font, otherwise the
		// document will not be rendered and an exception will be thrown.
		FontSettings.setUseSubstituteFont(true);

        // Check flag to show api
        if (args != null && args.length > 0 && "-hideapi".equalsIgnoreCase(args [0]))
        {
            m_ShowAPI = false;
        }

        SwingUtilities.invokeLater(new Runnable(){
			public void run()
			{
		        // Create and show the frame
		        PDFOptimizerFrame frame = new PDFOptimizerFrame();
		        frame.setVisible(true);
			}
        });
    }
    
    public void initialize()
    {
        this.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
        PDFOptimizerPanel pdfOptimizerPanel = new PDFOptimizerPanel();
        pdfOptimizerPanel.getjpAPI().setVisible(m_ShowAPI);
        this.setContentPane(pdfOptimizerPanel);
        this.setSize((int)(730 * SampleUtil.getDPIScalingMultiplier()), (int)(600 * SampleUtil.getDPIScalingMultiplier()));
        this.setLocationRelativeTo(null);
        this.setTitle("jPDFOptimizer Sample - " + PDFOptimizer.getVersion ());

        try
		{
    		// Try to set multiple images, java 1.6 only
			Method setIconImages = this.getClass().getMethod("setIconImages", new Class[] {List.class});
			Vector imageList = new Vector();
			imageList.add(ImageIO.read(getClass().getResourceAsStream("jPDFOptimizer16.png")));
			imageList.add(ImageIO.read(getClass().getResourceAsStream("jPDFOptimizer32.png")));
			setIconImages.invoke(this, new Object[] {imageList});
		}
		catch (Throwable t1)
		{
			try
	        {
	            this.setIconImage(ImageIO.read(getClass().getResourceAsStream("jPDFOptimizer16.png")));
	        }
	        catch(Throwable t2)
	        {
	        }
		}
    }

    private static void setLookAndFeel ()
    {
        // Set the look and feel
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Throwable t)
        {
        }
    }
}
