package jPDFOptimizerSamples;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.InputStream;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;
import javax.swing.text.Document;
import javax.swing.text.EditorKit;
import javax.swing.text.html.HTMLDocument;

import com.qoppa.pdf.IPassword;
import com.qoppa.pdf.PasswordDialog;
import com.qoppa.pdfOptimizer.ImageOutput;
import com.qoppa.pdfOptimizer.OptSettings;
import com.qoppa.pdfOptimizer.PDFOptimizer;

public class PDFOptimizerPanel extends JPanel implements KeyListener, IPassword
{
	// all visual components
	private JScrollPane jspRTFDoc = null;
	private JEditorPane jepRTFPane = null;
	private JPanel jpSamplePane = null;
	private File m_LastFileDir;
	private JButton jbOpen = null;
	private JButton jbSave = null;
	private JTextField jtfFileName = null;
	private JButton jbChangeOptions = null;
	private JLabel jlJARLocation = null;
	private JLabel jlSampleLocation = null;
	private JButton jbViewAPI = null;
	private JPanel jpAPI = null;
	private JPanel jpTitle = null;
	private JLabel jlTitle = null;
	private JRadioButton jrbSingleFile = null;
	private JRadioButton jrbDirectory = null;

	private PDFOptimizer m_PDFOptimizer;
	private OptSettings m_Optimization;

	private final static String OS_WINDOWS_START = "windows";
	private final static String OS_MAC = "mac";
	private final static String JAR_FILE_NAME = "jPDFOptimizer.jar";
	private final static String API_INDEX_FILENAME = "javadoc/index.html";
	private final static String SAMPLE_DIR_NAME = "jPDFOptimizerSamples";

	public PDFOptimizerPanel()
	{
		super();
		initialize();
		m_PDFOptimizer = null;
	}

	private void initialize()
	{
		// Layout
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		// Title panel
		this.add(getjpTitle(), null);
		this.add(Box.createRigidArea(new Dimension(3, 3)));

		// add rtf document scroll pane
		this.add(getjspRTFDoc(), null);
		this.add(Box.createRigidArea(new Dimension(3, 3)));

		// add api panel
		this.add(getjpAPI(), null);
		this.add(Box.createRigidArea(new Dimension(3, 3)));

		// add sample panel
		this.add(getjpSamplePane(), null);

		// add listeners
		getJtfFileName().addKeyListener(this);

		InputStream stream = getClass().getResourceAsStream("/jPDFOptimizer.html");
		if (stream != null)
		{
			try
			{
				getJepRTFPane().setContentType("text/html");

				EditorKit kit = getJepRTFPane().getEditorKit();
				Document doc = kit.createDefaultDocument();
				kit.read(stream, doc, 0);
				stream.close();
				getJepRTFPane().setDocument(doc);
				
				// Standard formatting is horrible, so use some stylesheet properties and system font
				Font font = new java.awt.Font("Dialog", java.awt.Font.PLAIN, new JLabel().getFont().getSize());
		        String rule = "body { font-family: " + font.getFamily() + "; " + "font-size: " + font.getSize() + "pt;}";
		        ((HTMLDocument)getJepRTFPane().getDocument()).getStyleSheet().addRule(rule);
		        rule = "h3 { font-family: " + font.getFamily() + "; " + "font-size: " + (int)(SampleUtil.getScaledFont(font.getSize(), 20)) + "pt;}";
				((HTMLDocument)getJepRTFPane().getDocument()).getStyleSheet().addRule(rule);
		        rule = "p { margin:0; padding:0;";
		        ((HTMLDocument)getJepRTFPane().getDocument()).getStyleSheet().addRule(rule);
			}
	        catch (Throwable t)
	        {
	            t.printStackTrace();
	        }
		}

		// Initialize help message
		File jarFile = new File("lib/" + JAR_FILE_NAME);
		getjlJARLocation().setText(JAR_FILE_NAME + " is located at " + jarFile.getAbsolutePath() + ".");
		getjlJARLocation().setToolTipText(jarFile.getAbsolutePath());

		File sampleDir = new File(SAMPLE_DIR_NAME);
		getjlSampleLocation().setText("Sample code is located at " + sampleDir.getAbsolutePath() + ".");
		getjlSampleLocation().setToolTipText(sampleDir.getAbsolutePath());
		
		// initialize opt settings
		// Instead of using OptSettings' defaults, change to the 'balanced' profile from Studio
		m_Optimization = new OptSettings();
		m_Optimization.setDiscardAltImages(true);
		m_Optimization.setDiscardPageThumbnails(true);
		m_Optimization.setFlattenAnnotations(true);
		m_Optimization.setCompressObjectsIntoStreams(true);

		Vector imageHandlers = new Vector();
		imageHandlers.add(new SampleImageHandler(SampleImageHandler.COLOR_FILTER, new Integer(150), new Integer(ImageOutput.CS_RETAIN), new Integer(ImageOutput.CO_JPEG), new Double(0.5)));
		imageHandlers.add(new SampleImageHandler(SampleImageHandler.GRAY_FILTER, new Integer(150), new Integer(ImageOutput.CS_RETAIN), new Integer(ImageOutput.CO_JPEG), new Double(0.5)));
		imageHandlers.add(new SampleImageHandler(SampleImageHandler.MONO_FILTER, new Integer(300), new Integer(ImageOutput.CS_RETAIN), new Integer(ImageOutput.CO_JBIG2), null));
		
		if (imageHandlers.size() > 0)
		{
			m_Optimization.setImageHandler(new CompositeImageHandler(imageHandlers));
		}
	}

	private JScrollPane getjspRTFDoc()
	{
		if (jspRTFDoc == null)
		{
			jspRTFDoc = new JScrollPane();
			jspRTFDoc.setPreferredSize(new Dimension((int)(681 * SampleUtil.getDPIScalingMultiplier()), (int)(300 * SampleUtil.getDPIScalingMultiplier())));
			jspRTFDoc.setViewportView(getJepRTFPane());
		}
		return jspRTFDoc;
	}

	private JEditorPane getJepRTFPane()
	{
		if (jepRTFPane == null)
		{
			jepRTFPane = new JEditorPane() {
				public void paint(Graphics g)
				{
					Graphics2D g2d = (Graphics2D) g;
					g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
					g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);

					super.paint(g);
				}
			};
			jepRTFPane.setEditable(false);
		}
		return jepRTFPane;
	}

	private JPanel getjpSamplePane()
	{
		if (jpSamplePane == null)
		{
			jpSamplePane = new JPanel();
			jpSamplePane.setLayout(new GridBagLayout());
			jpSamplePane.setBorder(BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.LOWERED), "Sample Processing",
							javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(new JLabel().getFont().getSize(), 12)),
							java.awt.Color.black));
			
			JPanel rbPanel = new JPanel();
			rbPanel.add(getJrbSingleFile());
			rbPanel.add(getJrbDirectory());

			JLabel fileLabel = new JLabel("File Name:");
			int fontSize = (int)SampleUtil.getScaledFont(fileLabel.getFont().getSize(), 12);
			fileLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, fontSize));

			JLabel customizeLabel = new JLabel("Change options for optimization.");
			customizeLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, fontSize));

			JLabel watermarkLabel = new JLabel("Watermark added in Demo Mode");
			watermarkLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, fontSize));
			watermarkLabel.setForeground(new Color(200, 0, 0));
			
			JLabel optimizeLabel = new JLabel("Optimize and Save.");
			optimizeLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, fontSize));
			
			JPanel lPanel = new JPanel();
			BoxLayout layout = new BoxLayout(lPanel, BoxLayout.X_AXIS);
			lPanel.setLayout(layout);
			lPanel.add(watermarkLabel);
			lPanel.add(Box.createHorizontalGlue());
			lPanel.add(optimizeLabel);

			GridBagConstraints constraints = new GridBagConstraints();
			constraints.insets = new Insets(1, 15, 1, 15);
			
			// put radio buttons in layout
			constraints.gridx = 1;
			constraints.gridy = 0;
			constraints.anchor = GridBagConstraints.LINE_START;
			jpSamplePane.add(rbPanel, constraints);
			
			// put file label in layout
			constraints.gridx = 0;
			constraints.gridy = 1;
			jpSamplePane.add(fileLabel, constraints);
			
			// put file text field in layout
			constraints.gridx = 1;
			constraints.gridy= 1;
			constraints.weightx = 1;
			constraints.fill = GridBagConstraints.HORIZONTAL;
			jpSamplePane.add(getJtfFileName(), constraints);
			
			// reset weight and fill
			constraints.fill = GridBagConstraints.NONE;
			constraints.weightx = 0;
			
			// put change options label in layout
			constraints.gridx = 1;
			constraints.gridy = 2;	
			constraints.anchor = GridBagConstraints.LINE_END;
			jpSamplePane.add(customizeLabel, constraints);
			
			// put demo and save labels in layout
			constraints.gridx = 1;
			constraints.gridy = 3;
			constraints.fill = GridBagConstraints.HORIZONTAL;
			jpSamplePane.add(lPanel, constraints);
			
			// put the buttons in the layout
			constraints.gridx = 2;
			constraints.gridy = 1;	
			constraints.fill = GridBagConstraints.HORIZONTAL;
			jpSamplePane.add(getJbOpen(), constraints);
			constraints.gridy = 2;
			jpSamplePane.add(getJbChangeOptions(), constraints);
			constraints.gridy = 3;
			constraints.insets = new Insets(1, 15, 15, 15);
			jpSamplePane.add(getJbSave(), constraints);

			ButtonGroup radioGroup = new ButtonGroup();
			radioGroup.add(getJrbSingleFile());
			radioGroup.add(getJrbDirectory());
			getJrbSingleFile().setSelected(true);
		}
		return jpSamplePane;
	}

	private JRadioButton getJrbSingleFile()
	{
		if (jrbSingleFile == null)
		{
			jrbSingleFile = new JRadioButton("Single File");
			jrbSingleFile.setBounds(new java.awt.Rectangle((int)(108 * SampleUtil.getDPIScalingMultiplier()), (int)(22 * SampleUtil.getDPIScalingMultiplier()), (int)(100 * SampleUtil.getDPIScalingMultiplier()), (int)(26 * SampleUtil.getDPIScalingMultiplier())));
			jrbSingleFile.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(jrbSingleFile.getFont().getSize(), 12)));
			jrbSingleFile.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					clearFields();
				}
			});
		}
		return jrbSingleFile;
	}
	
	private JRadioButton getJrbDirectory()
	{
		if (jrbDirectory == null)
		{
			jrbDirectory = new JRadioButton("Directory");
			jrbDirectory.setBounds(new java.awt.Rectangle((int)(208 * SampleUtil.getDPIScalingMultiplier()), (int)(22 * SampleUtil.getDPIScalingMultiplier()), (int)(100 * SampleUtil.getDPIScalingMultiplier()), (int)(26 * SampleUtil.getDPIScalingMultiplier())));
			jrbDirectory.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(jrbDirectory.getFont().getSize(), 12)));
			jrbDirectory.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					clearFields();
				}
			});
		}
		return jrbDirectory;
	}

	private void clearFields()
	{
		getJtfFileName().setText("");
	}

	private String getOptFileName(String originalName)
	{
		int split = originalName.lastIndexOf(".");
		if (split == -1)
		{
			split = originalName.length();
		}
		return originalName.substring(0, split) + "_opt" + originalName.substring(split, originalName.length());
	}

	protected File getFile(boolean open)
	{
		// Get first file name
		JFileChooser fileChooser = new JFileChooser();

		if (getJrbDirectory().isSelected())
		{
			fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		}

		// Initialize directory
		if (m_LastFileDir != null)
		{
			fileChooser.setCurrentDirectory(m_LastFileDir);

			if (!open && getFile() != null)
			{
				fileChooser.setSelectedFile(new File(getOptFileName(getFilePath())));
			}
		}

		// Look for PDF files
		fileChooser.setFileFilter(new PDFFileFilter());

		// Show the open dialog
		if (open)
		{
			if (fileChooser.showOpenDialog(this) == JFileChooser.CANCEL_OPTION)
			{
				return null;
			}
		}
		// Show save dialog
		else
		{
			if (fileChooser.showSaveDialog(this) == JFileChooser.CANCEL_OPTION)
			{
				return null;
			}
		}

		// Save last directory location
		File chosenFile = fileChooser.getSelectedFile();
		return chosenFile;
	}

	public String[] getPasswords()
	{
		// Create and show the password dialog
		Frame parentFrame = (Frame) SwingUtilities.windowForComponent(this);
		return PasswordDialog.showAndGetPassword(null, parentFrame);
	}

	private JButton getJbOpen()
	{
		if (jbOpen == null)
		{
			jbOpen = new JButton("Open");
			jbOpen.setBounds(new java.awt.Rectangle((int)(538 * SampleUtil.getDPIScalingMultiplier()), (int)(50 * SampleUtil.getDPIScalingMultiplier()), (int)(132 * SampleUtil.getDPIScalingMultiplier()), (int)(26 * SampleUtil.getDPIScalingMultiplier())));
			jbOpen.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(jbOpen.getFont().getSize(), 12)));
			jbOpen.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					// Open pdf file
					File file = getFile(true);
					if (file == null)
					{
						return;
					}
					open(file);
				}
			});
		}
		return jbOpen;
	}

	private JButton getJbSave()
	{
		if (jbSave == null)
		{
			jbSave = new JButton("Optimize");
			jbSave.setBounds(new java.awt.Rectangle((int)(538 * SampleUtil.getDPIScalingMultiplier()), (int)(105 * SampleUtil.getDPIScalingMultiplier()), (int)(132 * SampleUtil.getDPIScalingMultiplier()), (int)(26 * SampleUtil.getDPIScalingMultiplier())));
			jbSave.setFont(new Font("Dialog", Font.PLAIN, (int)SampleUtil.getScaledFont(jbSave.getFont().getSize(), 12)));
			jbSave.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					optimizeAndSave();
				}
			});
		}
		return jbSave;
	}

	public void optimizeAndSave()
	{
		final File selectedFile = getFile();
		if (selectedFile == null)
		{
			JOptionPane.showMessageDialog(null, "No document or directory open");
			return;
		}

		if (getJrbSingleFile().isSelected())
		{
			// get a file name
			final File file = getFile(false);
			if (file != null)
			{
		        // Check if it is the same document
		        if (file.equals(selectedFile))
		        {
		            JOptionPane.showMessageDialog(null, "Cannot save to the original location.");
					return;
		        }
				
				if (file.exists())
				{
					int rc = JOptionPane.showConfirmDialog(this, "Overwrite " + file.getName() + "?", "Overwrite", JOptionPane.YES_NO_OPTION);
					if (rc != JOptionPane.YES_OPTION)
					{
						return;
					}
				}

				Thread processThread = new Thread ()
				{
					public void run() 
					{
						final ProgressDialog progressDialog = ProgressDialog.getInstance((Frame) SwingUtilities.windowForComponent(PDFOptimizerPanel.this), "Optimize");

						try
						{
							progressDialog.getJbStopCancel().setVisible(false);
							progressDialog.setVisible(true);

							optimizeFile(selectedFile.getAbsolutePath(), file, progressDialog);
							progressDialog.dispose();
							JOptionPane.showMessageDialog(null, "Document optimized and saved at:\n" + file.getAbsolutePath());
						}
						catch (Throwable pdfException)
				        {
							progressDialog.dispose();
				        	JOptionPane.showMessageDialog(null, "Error saving file " + pdfException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				        	pdfException.printStackTrace();
				        }
					}
				};
						
				processThread.start();
			}
		}
		else
		{
			File dir = selectedFile;

			if (!dir.isDirectory())
			{
				JOptionPane.showMessageDialog(this, "Choose a valid directory");
			}
			
			final File[] fileList = dir.listFiles();

			Thread processThread = new Thread ()
			{
				public void run()
				{
					final ProgressDialog progressDialog = ProgressDialog.getInstance((Frame) SwingUtilities.windowForComponent(PDFOptimizerPanel.this), "Optimize");
					progressDialog.setVisible(true);

					int succeeded = 0;
					int failed = 0;
					Vector errors = new Vector();

					for (int i = 0; i < fileList.length && progressDialog.shouldContinue(); i++)
					{
						String name = fileList[i].getAbsolutePath();
						if (name.toLowerCase().endsWith(".pdf"))
						{
							try
							{
								progressDialog.setJlFunctionName("Optimize: " + fileList[i].getName());
								progressDialog.updateProgress("");
								optimizeFile(name, null, progressDialog);
								succeeded++;
							}
							catch (Throwable e)
							{
								e.printStackTrace();
								errors.add(name);
								errors.add(e.getMessage() == null? "null" : e.getMessage());
								failed++;
							}
						}
					}

					progressDialog.dispose();

					errors.add(new Integer(succeeded));

					if (succeeded + failed > 0)
					{
						ErrorDialogController errorController = new ErrorDialogController();
			            errorController.showDialog(errors);
					}
				}
			};
			processThread.start();
		}
	}

	private void optimizeFile(String absPath, File saveLoc, final ProgressDialog progressDialog) throws Throwable
	{
		try
		{
			m_PDFOptimizer = new PDFOptimizer(absPath, this);

			m_PDFOptimizer.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(final PropertyChangeEvent evt)
				{
					progressDialog.updateProgress(evt.getNewValue().toString());
				}
			});

			if (saveLoc == null)
			{
				saveLoc = new File(getOptFileName(absPath));
				if (saveLoc != null && saveLoc.exists())
				{
					int rc = JOptionPane.showConfirmDialog(this, "Overwrite " + saveLoc.getName() + "?", "Overwrite", JOptionPane.YES_NO_OPTION);
					if (rc != JOptionPane.YES_OPTION)
					{
						return;
					}
				}
			}
	
			m_PDFOptimizer.optimize(m_Optimization, saveLoc.getAbsolutePath());
		}
		catch (Throwable e)
		{
			saveLoc.delete();
			throw e;
		}
	}

	/**
	 * This method opens a pdf document and displays security information on the
	 * applet's components.
	 * 
	 */
	public void open(File file)
	{
		// keep file path for next time
		if (file != null)
		{
			m_LastFileDir = file.getParentFile();
		}

		// set file name in text field
		getJtfFileName().setText(file.getAbsolutePath());
	}

	public void changeOptions()
	{
		OptimizationOptionsController optimizationController = new OptimizationOptionsController(m_Optimization);
		Frame parentFrame = (Frame) SwingUtilities.windowForComponent(this);
		optimizationController.showDialog(parentFrame);
	}

	private JTextField getJtfFileName()
	{
		if (jtfFileName == null)
		{
			jtfFileName = new JTextField();
			jtfFileName.setBounds(new java.awt.Rectangle((int)(108 * SampleUtil.getDPIScalingMultiplier()), (int)(50 * SampleUtil.getDPIScalingMultiplier()), (int)(401 * SampleUtil.getDPIScalingMultiplier()), (int)(22 * SampleUtil.getDPIScalingMultiplier())));
		}
		return jtfFileName;
	}

	private JButton getJbChangeOptions()
	{
		if (jbChangeOptions == null)
		{
			jbChangeOptions = new JButton("Change Options");
			jbChangeOptions.setBounds(new java.awt.Rectangle((int)(538 * SampleUtil.getDPIScalingMultiplier()), (int)(78 * SampleUtil.getDPIScalingMultiplier()), (int)(132 * SampleUtil.getDPIScalingMultiplier()), (int)(26 * SampleUtil.getDPIScalingMultiplier())));
			jbChangeOptions.setFont(new Font("Dialog", Font.PLAIN, (int)SampleUtil.getScaledFont(jbChangeOptions.getFont().getSize(), 12)));
			jbChangeOptions.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					changeOptions();
				}
			});
		}
		return jbChangeOptions;
	}

	public void keyReleased(KeyEvent keyEvent)
	{
		// nothing to do
	}

	public void keyPressed(KeyEvent keyEvent)
	{
		// nothing to do
	}

	public void keyTyped(KeyEvent keyEvent)
	{
		if (keyEvent.getKeyChar() == KeyEvent.VK_ENTER)
		{
			open(new File(getJtfFileName().getText()));
		}
	}

	private JButton getJbViewAPI()
	{
		if (jbViewAPI == null)
		{
			jbViewAPI = new JButton("View API");
			jbViewAPI.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(jbViewAPI.getFont().getSize(), 12)));
			jbViewAPI.setBounds(new java.awt.Rectangle((int)(10 * SampleUtil.getDPIScalingMultiplier()), (int)(55 * SampleUtil.getDPIScalingMultiplier()), (int)(85 * SampleUtil.getDPIScalingMultiplier()), (int)(20 * SampleUtil.getDPIScalingMultiplier())));
			jbViewAPI.setMargin(new java.awt.Insets(2, 2, 2, 2));
			jbViewAPI.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					viewAPI();
				}
			});
		}
		return jbViewAPI;
	}

	private JLabel getjlJARLocation()
	{
		if (jlJARLocation == null)
		{
			jlJARLocation = new JLabel("JLabel");
			jlJARLocation.setBounds(new java.awt.Rectangle((int)(15 * SampleUtil.getDPIScalingMultiplier()), (int)(9 * SampleUtil.getDPIScalingMultiplier()), (int)(650 * SampleUtil.getDPIScalingMultiplier()), (int)(16 * SampleUtil.getDPIScalingMultiplier())));
			jlJARLocation.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(jlJARLocation.getFont().getSize(), 12)));
		}
		return jlJARLocation;
	}

	private JLabel getjlSampleLocation()
	{
		if (jlSampleLocation == null)
		{
			jlSampleLocation = new JLabel("JLabel");
			jlSampleLocation.setBounds(new java.awt.Rectangle((int)(15 * SampleUtil.getDPIScalingMultiplier()), (int)(30 * SampleUtil.getDPIScalingMultiplier()), (int)(650 * SampleUtil.getDPIScalingMultiplier()), (int)(16 * SampleUtil.getDPIScalingMultiplier())));
			jlSampleLocation.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(jlSampleLocation.getFont().getSize(), 12)));
		}
		return jlSampleLocation;
	}

	private boolean isSystemMac()
	{
		// Check the OS
		String osName = System.getProperty("os.name");
		osName = osName.toLowerCase();
		int firstIndexOfMac = osName.indexOf(OS_MAC);
		if (firstIndexOfMac == -1)
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	private boolean isSystemWindows()
	{
		// Check the OS
		String osName = System.getProperty("os.name");
		osName = osName.toLowerCase();
		return osName.startsWith(OS_WINDOWS_START);
	}

	protected void viewAPI()
	{
		try
		{
			File apiIndex = new File(API_INDEX_FILENAME);
			if (isSystemWindows())
			{
				Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + apiIndex.getAbsolutePath());
			}
			else if (isSystemMac())
			{
				String[] cmdArray = new String[2];
				cmdArray[0] = "open";
				cmdArray[1] = apiIndex.getAbsolutePath();
				Runtime.getRuntime().exec(cmdArray);
			}
			// assume Unix or Linux
			else
			{
				String[] browsers = { "firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape" };
				String browser = null;
				for (int count = 0; count < browsers.length && browser == null; count++)
				{
					if (Runtime.getRuntime().exec(new String[] { "which", browsers[count] }).waitFor() == 0)
					{
						browser = browsers[count];
						break;
					}
				}
				if (browser == null)
					throw new Exception("Could not find web browser");
				else
					Runtime.getRuntime().exec(new String[] { browser, apiIndex.getAbsolutePath() });
			}
		}
		catch (Throwable t)
		{
			javax.swing.JOptionPane.showMessageDialog(this, t.getMessage());
		}
	}

	public JPanel getjpAPI()
	{
		if (jpAPI == null)
		{
			jpAPI = new JPanel();
			jpAPI.setLayout(null);
			jpAPI.setPreferredSize(new Dimension((int)(681 * SampleUtil.getDPIScalingMultiplier()), (int)(150 * SampleUtil.getDPIScalingMultiplier())));
			jpAPI.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.LOWERED));
			JLabel jlViewAPI = new JLabel("to view the API.");
			jlViewAPI.setBounds(new java.awt.Rectangle((int)(100 * SampleUtil.getDPIScalingMultiplier()), (int)(57 * SampleUtil.getDPIScalingMultiplier()), (int)(126 * SampleUtil.getDPIScalingMultiplier()), (int)(16 * SampleUtil.getDPIScalingMultiplier())));
			jlViewAPI.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, (int)SampleUtil.getScaledFont(jlViewAPI.getFont().getSize(), 12)));
			jpAPI.add(getjlJARLocation(), null);
			jpAPI.add(jlViewAPI, null);
			jpAPI.add(getJbViewAPI(), null);
			jpAPI.add(getjlSampleLocation(), null);
		}
		return jpAPI;
	}

	public JPanel getjpTitle()
	{
		if (jpTitle == null)
		{
			jpTitle = new JPanel();
			// jpTitle.setLayout (new BorderLayout (5, 0));
			jpTitle.add(getjlTitle());// , BorderLayout.CENTER);
			jpTitle.setPreferredSize(new Dimension((int)(700 * SampleUtil.getDPIScalingMultiplier()), (int)(30 * SampleUtil.getDPIScalingMultiplier())));
			jpTitle.setBorder(new EtchedBorder());
		}
		return jpTitle;
	}

	public JLabel getjlTitle()
	{
		if (jlTitle == null)
		{
			jlTitle = new JLabel();
			jlTitle.setText(PDFOptimizer.getVersion());
			jlTitle.setFont(new Font("dialog", Font.BOLD, (int)SampleUtil.getScaledFont(jlTitle.getFont().getSize(), 14)));
		}
		return jlTitle;
	}
	
	private String getFilePath()
	{
		return getJtfFileName().getText();
	}
	
	private File getFile()
	{
		File f = new File(getFilePath());
		if (!f.exists())
		{
			return null;
		}
		return f;
	}

	public class ErrorsDialog extends JDialog
	{
		private JPanel jpErrors = null;
		private JScrollPane jspErrors = null;
		private JTextArea jtaErrors = null;
		private JButton jbOK = null;
		
		public ErrorsDialog(JFrame parent)
		{
			super(parent);
			initialize();
		}
		
		public JButton getJbOk()
		{
			if (jbOK == null)
			{
				jbOK = new JButton();
				jbOK.setName("ButtonOK");
				jbOK.setText("Ok");
			}
			return jbOK;
		}
		
		public JTextArea getJtaErrors()
		{
			if (jtaErrors == null)
			{
				jtaErrors = new JTextArea();
				jtaErrors.setEditable(false);
			}
			return jtaErrors;
		}
		
		private JPanel getJpErrors()
		{
			if (jpErrors == null)
			{
				jpErrors = new JPanel();
				jpErrors.add(getJspErrors());
				jpErrors.add(getJbOk());
			}
			return jpErrors;
		}
		
		private JScrollPane getJspErrors()
		{
			if (jspErrors == null)
			{
				jspErrors = new JScrollPane();
				jspErrors.setViewportView(getJtaErrors());
			}
			return jspErrors;
		}
		
		private void initialize()
		{
			this.setModal(true);
			this.getRootPane().setDefaultButton(getJbOk());
			setContentPane(getJpErrors());
			this.pack();
		}
	}

	public class ErrorDialogController implements ActionListener
	{
		private String CMD_OK = "OK";
		
		private ErrorsDialog m_Dialog;

		public void actionPerformed(ActionEvent e)
		{
			if (e.getActionCommand() == CMD_OK)
			{
				m_Dialog.dispose();
			}
		}
		
		protected void showDialog(Vector info)
		{
			m_Dialog = new ErrorsDialog(null);

			// action commands
			m_Dialog.getJbOk().addActionListener(this);
			m_Dialog.getJbOk().setActionCommand(CMD_OK);
			
			// Prep the info for the dialog
			// 
			int succeeded = toInteger(info.remove(info.size() - 1));
			String message = "";
			if (succeeded > 0)
			{
				message += "There were " + succeeded + " file(s) optimized successfully and saved in " + getFile().getName() + ".  ";
			}
			if (info.size() > 0)
			{
				message += info.size()/2 + " file(s) failed to optimize";
				message += "\nFailed Documents:";
			}
			
			for (int i = 0; i < info.size(); i++)
			{
				String name = info.get(i).toString();
				i++;
				String error = info.get(i).toString();
				message += "\n" + name + "- " + error;
			}
			m_Dialog.getJtaErrors().setText(message);

			m_Dialog.pack();

			m_Dialog.setLocationRelativeTo(null);

			// Show the dialog
			m_Dialog.setVisible(true);
		}
		
		private int toInteger(Object obj)
		{
			if (obj != null)
			{
				if (obj instanceof Number)
				{
					return ((Number)obj).intValue();
				}
				try
				{
					return Double.valueOf (obj.toString ()).intValue ();
				}
				catch (Exception e)
				{
					return 0;
				}
			}
			
			return 0;
		}
	}
}

