package jPDFOptimizerSamples;

import com.qoppa.pdfOptimizer.ImageHandler;
import com.qoppa.pdfOptimizer.ImageInfo;
import com.qoppa.pdfOptimizer.ImageOutput;

public class SampleImageHandler implements ImageHandler
{	
	public static int COLOR_FILTER = 1;
	public static int GRAY_FILTER = 2;
	public static int MONO_FILTER = 3;
	
	private int mFilter;
	private Integer mMaxDpi;
	private Integer mColorSpace = new Integer(ImageOutput.CS_RETAIN);
	private Integer mCompression = new Integer(ImageOutput.CO_RETAIN);
	private Double mQuality;
	
	public SampleImageHandler(int filter, Integer maxDpi, Integer colorSpace, Integer compression, Double quality)
	{
		mFilter = filter;
		mMaxDpi = maxDpi;
		mColorSpace = colorSpace;
		mCompression = compression;
		mQuality = quality;
	}

	public ImageOutput convertImage(ImageInfo imageInfo)
	{		
		if (filter(imageInfo))
		{
			// defaults
			int newWidth = imageInfo.getImageWidth();
			int newHeight = imageInfo.getImageHeight();
			
			if (mMaxDpi != null)
			{
				int maxDPI = mMaxDpi.intValue();
				if (imageInfo.getDPIX() > maxDPI || imageInfo.getDPIY() > maxDPI)
				{
					// Calculate new dimensions to match DPI
					float scale = Math.min(maxDPI / imageInfo.getDPIX(), maxDPI / imageInfo.getDPIY());
					newWidth = (int)Math.max(1, imageInfo.getImageWidth() * scale + 0.5);
					newHeight = (int)Math.max(1, imageInfo.getImageHeight() * scale + 0.5);
				}
			}
			
			ImageOutput output = new ImageOutput(mCompression.intValue(), mColorSpace.intValue(), newWidth, newHeight);
			
			if (mQuality != null)
			{
				output.setCompressionQuality(mQuality.floatValue());
			}
			
			return output;
		}
		return null;
	}
	
	private boolean filter(ImageInfo imageInfo)
	{
		if (mFilter == COLOR_FILTER)
		{
			return imageInfo.isColor();
		}
		else if (mFilter == GRAY_FILTER)
		{
			return imageInfo.isGray();
		}
		else if (mFilter == MONO_FILTER)
		{
			return imageInfo.isMonochrome();
		}

		return true;
	}

	public int getFilter()
	{
		return mFilter;
	}

	public Integer getMaxDpi()
	{
		return mMaxDpi;
	}

	public Integer getColorSpace()
	{
		return mColorSpace;
	}

	public Integer getCompression()
	{
		return mCompression;
	}

	public Double getQuality()
	{
		return mQuality;
	}
}
